@@ -2,16 +2,14 @@
  * dvbhdffdevice.h: The DVB HD Full Featured device interface
  *
  * See the README file for copyright information and how to reach the author.
- *
- * $Id: dvbhdffdevice.h 1.7 2011/09/10 10:17:32 kls Exp $
  */
 
 #ifndef __DVBHDFFDEVICE_H
 #define __DVBHDFFDEVICE_H
 
 #include "hdffcmd.h"
-#include "vdr/dvbdevice.h"
-#include "vdr/dvbspu.h"
+#include <vdr/dvbdevice.h>
+#include <vdr/dvbspu.h>
 
 /// The cDvbHdFfDevice implements a DVB device which can be accessed through the Linux DVB driver API.
 
@@ -81,6 +79,7 @@ private:
   int playAudioPid;
   bool freezed;
   bool trickMode;
+  bool isPlayingVideo;
 
   // Pes2Ts conversion stuff
   uint8_t videoCounter;
@@ -98,6 +97,8 @@ protected:
   virtual int PlayTsAudio(const uchar *Data, int Length);
 public:
   virtual int64_t GetSTC(void);
+  virtual cRect CanScaleVideo(const cRect &Rect, int Alignment = taCenter);
+  virtual void ScaleVideo(const cRect &Rect = cRect::Null);
   virtual void TrickSpeed(int Speed);
   virtual void Clear(void);
   virtual void Play(void);
