@@ -4,32 +4,36 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: cutter.h 1.1 2002/06/22 10:03:15 kls Exp $
+ * $Id: cutter.h 2.3 2012/02/16 12:05:33 kls Exp $
  */
 
 #ifndef __CUTTER_H
 #define __CUTTER_H
 
+#include "thread.h"
+#include "tools.h"
+
 class cCuttingThread;
-#ifdef USE_CUTTERQUEUE
-class cMutex;
-#endif /* CUTTERQUEUE */
 
 class cCutter {
 private:
-  static char *editedVersionName;
+  static cMutex mutex;
+  static cString originalVersionName;
+  static cString editedVersionName;
   static cCuttingThread *cuttingThread;
   static bool error;
   static bool ended;
-#ifdef USE_CUTTERQUEUE
-  static cMutex *cutterLock;
-#endif /* CUTTERQUEUE */
 public:
   static bool Start(const char *FileName);
   static void Stop(void);
-  static bool Active(void);
+  static bool Active(const char *FileName = NULL);
+         ///< Returns true if the cutter is currently active.
+         ///< If a FileName is given, true is only returned if either the
+         ///< original or the edited file name is equal to FileName.
   static bool Error(void);
   static bool Ended(void);
   };
 
+bool CutRecording(const char *FileName);
+
 #endif //__CUTTER_H
