@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbplayer.c 2.27 2012/05/06 11:02:35 kls Exp $
+ * $Id: dvbplayer.c 2.35 2013/03/08 13:44:19 kls Exp $
  */
 
 #include "dvbplayer.h"
@@ -33,6 +33,7 @@ private:
 public:
   cPtsIndex(void);
   void Clear(void);
+  bool IsEmpty(void);
   void Put(uint32_t Pts, int Index);
   int FindIndex(uint32_t Pts);
   };
@@ -49,6 +50,12 @@ void cPtsIndex::Clear(void)
   w = r = 0;
 }
 
+bool cPtsIndex::IsEmpty(void)
+{
+  cMutexLock MutexLock(&mutex);
+  return w == r;
+}
+
 void cPtsIndex::Put(uint32_t Pts, int Index)
 {
   cMutexLock MutexLock(&mutex);
@@ -220,6 +227,7 @@ private:
   cFrame *readFrame;
   cFrame *playFrame;
   cFrame *dropFrame;
+  bool resyncAfterPause;
   void TrickSpeed(int Increment);
   void Empty(void);
   bool NextFile(uint16_t FileNumber = 0, off_t FileOffset = -1);
@@ -240,6 +248,7 @@ public:
   void SkipSeconds(int Seconds);
   void Goto(int Position, bool Still = false);
   virtual double FramesPerSecond(void) { return framesPerSecond; }
+  virtual void SetAudioTrack(eTrackType Type, const tTrackId *TrackId);
   virtual bool GetIndex(int &Current, int &Total, bool SnapToIFrame = false);
   virtual bool GetReplayMode(bool &Play, bool &Forward, int &Speed);
   };
@@ -270,6 +279,7 @@ cDvbPlayer::cDvbPlayer(const char *FileName, bool PauseLive)
   readFrame = NULL;
   playFrame = NULL;
   dropFrame = NULL;
+  resyncAfterPause = false;
   isyslog("replay %s", FileName);
   fileName = new cFileName(FileName, false, false, isPesRecording);
   replayFile = fileName->Open();
@@ -513,7 +523,7 @@ void cDvbPlayer::Action(void)
 
           if (dropFrame) {
              if (!eof || (playDir != pdForward && dropFrame->Index() > 0) || (playDir == pdForward && dropFrame->Index() < readIndex)) {
-                ringBuffer->Drop(dropFrame); // the very first and last frame are continously repeated to flush data through the device
+                ringBuffer->Drop(dropFrame); // the very first and last frame are continuously repeated to flush data through the device
                 dropFrame = NULL;
                 }
              }
@@ -548,10 +558,11 @@ void cDvbPlayer::Action(void)
                 }
              if (p) {
                 int w;
+                bool VideoOnly = (dropFrame || playMode != pmPlay && !(playMode == pmSlow && playDir == pdForward)) && DeviceIsPlayingVideo();
                 if (isPesRecording)
-                   w = PlayPes(p, pc, playMode != pmPlay && !(playMode == pmSlow && playDir == pdForward) && DeviceIsPlayingVideo());
+                   w = PlayPes(p, pc, VideoOnly);
                 else
-                   w = PlayTs(p, pc, playMode != pmPlay && !(playMode == pmSlow && playDir == pdForward) && DeviceIsPlayingVideo());
+                   w = PlayTs(p, pc, VideoOnly);
                 if (w > 0) {
                    p += w;
                    pc -= w;
@@ -575,7 +586,7 @@ void cDvbPlayer::Action(void)
           if (eof || SwitchToPlayFrame) {
              bool SwitchToPlay = false;
              uint32_t Stc = DeviceGetSTC();
-             if (Stc != LastStc)
+             if (Stc != LastStc || playMode == pmPause)
                 StuckAtEof = 0;
              else if (!StuckAtEof)
                 StuckAtEof = time(NULL);
@@ -635,7 +646,13 @@ void cDvbPlayer::Play(void)
      DevicePlay();
      playMode = pmPlay;
      playDir = pdForward;
-    }
+     if (resyncAfterPause) {
+        int Current, Total;
+        if (GetIndex(Current, Total, true))
+           Goto(Current);
+        resyncAfterPause = false;
+        }
+     }
 }
 
 void cDvbPlayer::Forward(void)
@@ -798,6 +815,21 @@ void cDvbPlayer::Goto(int Index, bool Still)
      }
 }
 
+void cDvbPlayer::SetAudioTrack(eTrackType Type, const tTrackId *TrackId)
+{
+  if (!cThread::IsMainThread())
+     return; // only do this upon user interaction
+  if (playMode == pmPlay) {
+     if (!ptsIndex.IsEmpty()) {
+        int Current, Total;
+        if (GetIndex(Current, Total, true))
+           Goto(Current);
+        }
+     }
+  else if (playMode == pmPause)
+     resyncAfterPause = true;
+}
+
 bool cDvbPlayer::GetIndex(int &Current, int &Total, bool SnapToIFrame)
 {
   if (index) {
