@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: eitscan.h 2.0 2005/09/04 10:51:35 kls Exp $
+ * $Id: eitscan.h 2.1 2012/03/07 13:54:16 kls Exp $
  */
 
 #ifndef __EITSCAN_H
@@ -24,7 +24,6 @@ private:
          ScanTimeout = 20
        };
   time_t lastScan, lastActivity;
-  cDevice *currentDevice;
   int currentChannel;
   cScanList *scanList;
   cTransponderList *transponderList;
@@ -32,7 +31,6 @@ public:
   cEITScanner(void);
   ~cEITScanner();
   bool Active(void) { return currentChannel || lastActivity == 0; }
-  bool UsesDevice(const cDevice *Device) { return currentDevice == Device; }
   void AddTransponder(cChannel *Channel);
   void ForceScan(void);
   void Activity(void);
