@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menu.h 2.9 2012/05/12 11:48:04 kls Exp $
+ * $Id: menu.h 2.13 2012/12/07 13:44:13 kls Exp $
  */
 
 #ifndef __MENU_H
@@ -187,6 +187,7 @@ public:
   };
 
 cOsdObject *CamControl(void);
+bool CamMenuActive(void);
 
 class cMenuRecordingItem;
 
@@ -203,9 +204,10 @@ private:
   eOSState Rewind(void);
   eOSState Delete(void);
   eOSState Info(void);
+  eOSState Sort(void);
   eOSState Commands(eKeys Key = kNone);
 protected:
-  cRecording *GetRecording(cMenuRecordingItem *Item);
+  cString DirectoryName(void);
 public:
   cMenuRecordings(const char *Base = NULL, int Level = 0, bool OpenSubMenus = false);
   ~cMenuRecordings();
@@ -257,6 +259,7 @@ class cReplayControl : public cDvbPlayerControl {
 private:
   cSkinDisplayReplay *displayReplay;
   cMarks marks;
+  bool marksModified;
   bool visible, modeOnly, shown, displayFrames;
   int lastCurrent, lastTotal;
   bool lastPlay, lastForward;
