@@ -4,31 +4,17 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: receiver.c 2.4 2010/12/12 23:16:25 kls Exp $
+ * $Id: receiver.c 2.7 2012/06/02 13:20:38 kls Exp $
  */
 
 #include "receiver.h"
 #include <stdio.h>
 #include "tools.h"
 
-#ifdef LEGACY_CRECEIVER
-cReceiver::cReceiver(tChannelID ChannelID, int Priority, int Pid, const int *Pids1, const int *Pids2, const int *Pids3)
-{
-  device = NULL;
-  channelID = ChannelID;
-  priority = Priority;
-  numPids = 0;
-  AddPid(Pid);
-  AddPids(Pids1);
-  AddPids(Pids2);
-  AddPids(Pids3);
-}
-#endif
-
 cReceiver::cReceiver(const cChannel *Channel, int Priority)
 {
   device = NULL;
-  priority = Priority;
+  priority = constrain(Priority, MINPRIORITY, MAXPRIORITY);
   numPids = 0;
   SetPids(Channel);
 }
@@ -83,7 +69,7 @@ bool cReceiver::SetPids(const cChannel *Channel)
             AddPids(Channel->Dpids()) &&
             AddPids(Channel->Spids());
      }
-return true;
+  return true;
 }
 
 bool cReceiver::WantsPid(int Pid)
