@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: remote.c 2.5 2012/01/16 16:57:00 kls Exp $
+ * $Id: remote.c 2.8 2013/02/03 15:44:55 kls Exp $
  */
 
 #include "remote.h"
@@ -295,6 +295,14 @@ int cKbdRemote::MapCodeToFunc(uint64_t Code)
   return kfNone;
 }
 
+void cKbdRemote::PutKey(uint64_t Code, bool Repeat, bool Release)
+{
+  if (rawMode || !Put(Code, Repeat, Release)) {
+     if (int func = MapCodeToFunc(Code))
+        Put(KBDKEY(func), Repeat, Release);
+     }
+}
+
 int cKbdRemote::ReadKey(void)
 {
   cPoller Poller(STDIN_FILENO);
@@ -356,14 +364,56 @@ uint64_t cKbdRemote::ReadKeySequence(void)
 
 void cKbdRemote::Action(void)
 {
+  cTimeMs FirstTime;
+  cTimeMs LastTime;
+  uint64_t FirstCommand = 0;
+  uint64_t LastCommand = 0;
+  bool Delayed = false;
+  bool Repeat = false;
+
   while (Running()) {
         uint64_t Command = ReadKeySequence();
         if (Command) {
-           if (rawMode || !Put(Command)) {
-              int func = MapCodeToFunc(Command);
-              if (func)
-                 Put(KBDKEY(func));
+           if (Command == LastCommand) {
+              // If two keyboard events with the same command come in without an intermediate
+              // timeout, this is a long key press that caused the repeat function to kick in:
+              Delayed = false;
+              FirstCommand = 0;
+              if (FirstTime.Elapsed() < (uint)Setup.RcRepeatDelay)
+                 continue; // repeat function kicks in after a short delay
+              if (LastTime.Elapsed() < (uint)Setup.RcRepeatDelta)
+                 continue; // skip same keys coming in too fast
+              PutKey(Command, true);
+              Repeat = true;
+              LastTime.Set();
+              }
+           else if (Command == FirstCommand) {
+              // If the same command comes in twice with an intermediate timeout, we
+              // need to delay the second command to see whether it is going to be
+              // a repeat function or a separate key press:
+              Delayed = true;
+              }
+           else {
+              // This is a totally new key press, so we accept it immediately:
+              PutKey(Command);
+              Delayed = false;
+              FirstCommand = Command;
+              FirstTime.Set();
               }
            }
+        else if (Repeat) {
+           // Timeout after a repeat function, so we generate a 'release':
+           PutKey(LastCommand, false, true);
+           Repeat = false;
+           }
+        else if (Delayed && FirstCommand) {
+           // Timeout after two normal key presses of the same key, so accept the
+           // delayed key:
+           PutKey(FirstCommand);
+           Delayed = false;
+           FirstCommand = 0;
+           FirstTime.Set();
+           }
+        LastCommand = Command;
         }
 }
