@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: skins.c 2.3 2011/08/21 11:21:19 kls Exp $
+ * $Id: skins.c 2.10 2012/06/02 11:44:14 kls Exp $
  */
 
 #include "skins.h"
@@ -67,9 +67,15 @@ cSkinDisplay::~cSkinDisplay()
 
 cSkinDisplayMenu::cSkinDisplayMenu(void)
 {
+  menuCategory = mcUndefined;
   SetTabs(0);
 }
 
+void cSkinDisplayMenu::SetMenuCategory(eMenuCategory MenuCategory)
+{
+  menuCategory = MenuCategory;
+}
+
 void cSkinDisplayMenu::SetTabs(int Tab1, int Tab2, int Tab3, int Tab4, int Tab5)
 {
   tabs[0] = 0;
@@ -78,9 +84,8 @@ void cSkinDisplayMenu::SetTabs(int Tab1, int Tab2, int Tab3, int Tab4, int Tab5)
   tabs[3] = Tab3 ? tabs[2] + Tab3 : 0;
   tabs[4] = Tab4 ? tabs[3] + Tab4 : 0;
   tabs[5] = Tab5 ? tabs[4] + Tab5 : 0;
-  int AvgCharWidth = Setup.FontOsdSize * 3 / 5; // just an estimate
   for (int i = 1; i < MaxTabs; i++)
-      tabs[i] *= AvgCharWidth;
+      tabs[i] *= AvgCharWidth();
 }
 
 void cSkinDisplayMenu::Scroll(bool Up, bool Page)
@@ -167,6 +172,11 @@ cSkinDisplayReplay::cSkinDisplayReplay(void)
   marks = NULL;
 }
 
+void cSkinDisplayReplay::SetRecording(const cRecording *Recording)
+{
+  SetTitle(Recording->Title());
+}
+
 void cSkinDisplayReplay::SetMarks(const cMarks *Marks)
 {
   marks = Marks;
