@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: status.c 2.0 2008/02/16 14:46:31 kls Exp $
+ * $Id: status.c 2.1 2012/03/07 14:17:24 kls Exp $
  */
 
 #include "status.h"
@@ -29,10 +29,10 @@ void cStatus::MsgTimerChange(const cTimer *Timer, eTimerChange Change)
       sm->TimerChange(Timer, Change);
 }
 
-void cStatus::MsgChannelSwitch(const cDevice *Device, int ChannelNumber)
+void cStatus::MsgChannelSwitch(const cDevice *Device, int ChannelNumber, bool LiveView)
 {
   for (cStatus *sm = statusMonitors.First(); sm; sm = statusMonitors.Next(sm))
-      sm->ChannelSwitch(Device, ChannelNumber);
+      sm->ChannelSwitch(Device, ChannelNumber, LiveView);
 }
 
 void cStatus::MsgRecording(const cDevice *Device, const char *Name, const char *FileName, bool On)
