@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: svdrp.h 2.1 2011/12/04 13:51:44 kls Exp $
+ * $Id: svdrp.h 2.3 2012/04/26 10:30:06 kls Exp $
  */
 
 #ifndef __SVDRP_H
@@ -43,7 +43,7 @@ class cSVDRP {
 private:
   cSocket socket;
   cFile file;
-  cRecordings Recordings;
+  cRecordings recordings;
   cPUTEhandler *PUTEhandler;
   int numChars;
   int length;
@@ -71,7 +71,6 @@ private:
   void CmdMODC(const char *Option);
   void CmdMODT(const char *Option);
   void CmdMOVC(const char *Option);
-  void CmdMOVT(const char *Option);
   void CmdNEWC(const char *Option);
   void CmdNEWT(const char *Option);
   void CmdNEXT(const char *Option);
