@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: tools.c 2.24 2012/05/12 13:29:20 kls Exp $
+ * $Id: tools.c 2.29 2012/12/08 11:16:30 kls Exp $
  */
 
 #include "tools.h"
@@ -18,6 +18,7 @@ extern "C" {
 #include <jpeglib.h>
 #undef boolean
 }
+#include <locale.h>
 #include <stdlib.h>
 #include <sys/time.h>
 #include <sys/vfs.h>
@@ -285,11 +286,56 @@ int64_t StrToNum(const char *s)
   return n;
 }
 
+bool StrInArray(const char *a[], const char *s)
+{
+  if (a) {
+     while (*a) {
+           if (strcmp(*a, s) == 0)
+              return true;
+           a++;
+           }
+     }
+  return false;
+}
+
 cString AddDirectory(const char *DirName, const char *FileName)
 {
   return cString::sprintf("%s/%s", DirName && *DirName ? DirName : ".", FileName);
 }
 
+#define DECIMAL_POINT_C '.'
+
+double atod(const char *s)
+{
+  static lconv *loc = localeconv();
+  if (*loc->decimal_point != DECIMAL_POINT_C) {
+     char buf[strlen(s) + 1];
+     char *p = buf;
+     while (*s) {
+           if (*s == DECIMAL_POINT_C)
+              *p = *loc->decimal_point;
+           else
+              *p = *s;
+           p++;
+           s++;
+           }
+     *p = 0;
+     return atof(buf);
+     }
+  else
+     return atof(s);
+}
+
+cString dtoa(double d, const char *Format)
+{
+  static lconv *loc = localeconv();
+  char buf[16];
+  snprintf(buf, sizeof(buf), Format, d);
+  if (*loc->decimal_point != DECIMAL_POINT_C)
+     strreplace(buf, *loc->decimal_point, DECIMAL_POINT_C);
+  return buf;
+}
+
 cString itoa(int n)
 {
   char buf[16];
@@ -433,8 +479,9 @@ bool RemoveFileOrDir(const char *FileName, bool FollowSymlinks)
   return true;
 }
 
-bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis)
+bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis, const char *IgnoreFiles[])
 {
+  bool HasIgnoredFiles = false;
   cReadDir d(DirName);
   if (d.Ok()) {
      bool empty = true;
@@ -445,9 +492,11 @@ bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis)
               struct stat st;
               if (stat(buffer, &st) == 0) {
                  if (S_ISDIR(st.st_mode)) {
-                    if (!RemoveEmptyDirectories(buffer, true))
+                    if (!RemoveEmptyDirectories(buffer, true, IgnoreFiles))
                        empty = false;
                     }
+                 else if (RemoveThis && IgnoreFiles && StrInArray(IgnoreFiles, e->d_name))
+                    HasIgnoredFiles = true;
                  else
                     empty = false;
                  }
@@ -458,6 +507,19 @@ bool RemoveEmptyDirectories(const char *DirName, bool RemoveThis)
               }
            }
      if (RemoveThis && empty) {
+        if (HasIgnoredFiles) {
+           while (*IgnoreFiles) {
+                 cString buffer = AddDirectory(DirName, *IgnoreFiles);
+                 if (access(buffer, F_OK) == 0) {
+                    dsyslog("removing %s", *buffer);
+                    if (remove(buffer) < 0) {
+                       LOG_ERROR_STR(*buffer);
+                       return false;
+                       }
+                    }
+                 IgnoreFiles++;
+                 }
+           }
         dsyslog("removing %s", DirName);
         if (remove(DirName) < 0) {
            LOG_ERROR_STR(DirName);
@@ -814,7 +876,8 @@ cCharSetConv::cCharSetConv(const char *FromCode, const char *ToCode)
 cCharSetConv::~cCharSetConv()
 {
   free(result);
-  iconv_close(cd);
+  if (cd != (iconv_t)-1)
+     iconv_close(cd);
 }
 
 void cCharSetConv::SetSystemCharacterTable(const char *CharacterTable)
